__version__ = "0.3.2"

from arithmetic_eval.utils import evaluate
from arithmetic_eval.exceptions import MaliciousInputError

__all__ = [
    "__version__",
    "evaluate",
    "MaliciousInputError",
]
